;;-----------------------------LICENSE NOTICE------------------------------------
;; _________.__             .__                   __        .___    ___.    __   
;; \__    ___/|  |__   ____   |  | _____    _______/  |_    __| _/____\_ |___/  |_ 
;;   |    |   |  |  \_/ __ \  |  | \__  \  /  ___/\   __\  / __ |/ __ \| __ \   __\
;;   |    |   |   Y  \  ___/  |  |__/ __ \_\___ \  |  |   / /_/ \  ___/| \_\ \  |  
;;   |____|   |___|  /\___  > |____(____  /____  > |__|   \____ |\___  >___  /__|  
;;                 \/     \/            \/     \/              \/    \/    \/      
;;
;;  Code Copyright (C) 2018 RetroJAM Games (@RetroJamGames)
;;  This file is part of The Last Debt: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

;;====================
;; GLOBAL: Functions
;;====================  
.globl ent_update
.globl ent_moveRight 
.globl ent_moveLeft
.globl ent_clear


;;====================
;; MACRO: Generic entity
;;====================
.macro DefineDrawableEntity _name, _x, _y, _w, _h, _sp, _vw
 _name:
  .db _vw
  .db _x, _y
  .db _w, _h 
  .dw _sp
.endm
;;============================
;; MACRO: Obstacle entity
;;============================
.macro DefineObstacle _name, _x, _y,  _w, _h, _tel, _vw, _sp
_name:
  .db _vw
   .db    _x, _y    ;; X, Y
   .db    _w, _h     ;; W, H
   .dw   _sp
   .db   _tel        ;; Teleport
.endm

obs_sp_l = 5
obs_sp_h = 6
h_tel = 7

;;============================
;; MACRO: Wall entity
;;============================
.macro DefineWall _name, _x, _y, _w, _h, _vw, _sp
_name:
  .db _vw
  .db    _x, _y    ;; X, Y
  .db    _w, _h     ;; W, H
  .dw _sp
.endm

;;;;;;;;;;;;;;;;;;;
;; CONSTANTS
;;;;;;;;;;;;;;;;;;;
  h_vw = 0
  h_x = 1
  h_y = 2
  h_w = 3
  h_h = 4
  h_sp_l = 5 
  h_sp_h = 6
;;==============================
;; MACRO: Bullet entity
;;==============================
.macro DefineBullet _name, _x, _y,  _w, _h,_sp, _upd, _dir, _type, _vw, _back
_name:
   DefineDrawableEntity _name'_dy, _x, _y, _w, _h, _sp, _vw
   .dw _upd
   .db _dir ;; if 0 right, 1 = left
   .db _type ;; enemy 1, hero 0
   .dw _back
.endm

.macro DefineBulletDefault _name, _suf
   DefineBullet _name'_suf, 0, 0, 0, 0,0x000, 0x0000, 0, 0, 0, 0x0000
.endm
.macro DefineNBullets _name, _n
  _c = 0
  .rept _n
     DefineBulletDefault _name, \_c
     _c = _c + 1
  .endm
.endm

;;;;;;;;;;;;;;;;;
;;CONSTANTS
;;;;;;;;;;;;;;;;;
b_up_l = 7 
b_up_h = 8
b_dir = 9
b_type = 10
b_back_l = 11
b_back_h = 12

;;=============================
;; MACRO: Enemy entity
;;==============================



.macro DefineEnemy _name, _x, _y, _w, _h, _sp, _upd, _shoot, _dir, _vx, _xi, _xf, _yi, _yf, _isUp, _health, _vw, _ovw, _ohealth, _scd, _sst, _px, _py, _ox, _oy
_name:
  DefineDrawableEntity _name'_de, _x, _y, _w, _h, _sp, _vw
    .dw   _upd      ;; Update 
    .db   _dir      ;; direccion a la que mira el enemigo 1-derecha y 0-izquierda
    .db   _vx       ;; Velocidad en el eje x
    .db   _xi       ;; Inicio del camino en el eje x
    .db   _xf       ;; Fin del camino en el eje x
    .db   _yi       ;; Distancia de la caida del enemigo
    .db   _yf       ;; Distancia del salto del enemigo
    .db   _isUp     ;; 0 si el enemigo esta en el primer piso y 1 si esta en el segundo
    .db _health     ;; vidas del enemigo
    .db _ovw        ;; Original View
    .db _ohealth    ;; Original Health
    .db _scd
    .db _sst
    .dw _shoot
    .db _px
    .db _py
    .db _ox
    .db _oy
.endm

h_dir = 9
e_vx = 10
e_xi = 11
e_xf = 12
e_yi = 13
e_yf = 14
e_isUp = 15
e_health = 16
e_ovw = 17
e_ohealth = 18
e_scd = 19
e_sst = 20
e_shl = 21
e_shh = 22
e_px = 23
e_py = 24
e_ox = 25
e_oy = 26





;;======================================
;; MACRO: Object entity
;;=======================================
.macro DefineObject _name, _x, _y, _w, _h, _sp, _type, _vw, _st
_name:
  DefineDrawableEntity _name'_dh, _x, _y, _w, _h, _sp, _vw
  .db   _type       ;; Type of object
  .db  _st          ;; 1=Ready to pick 0= picked
.endm

.macro DefineObjectDefault _name, _suf
   DefineObject _name'_suf, 0, 0, 0, 0, 0x00000, 0, 0, 0
.endm
.macro DefineNObjects _name, _n
  _c = 0
  .rept _n
     DefineObjectDefault _name, \_c
     _c = _c + 1
  .endm
.endm

o_type = 7
o_status = 8
