;;-----------------------------LICENSE NOTICE------------------------------------
;; _________.__             .__                   __        .___    ___.    __   
;; \__    ___/|  |__   ____   |  | _____    _______/  |_    __| _/____\_ |___/  |_ 
;;   |    |   |  |  \_/ __ \  |  | \__  \  /  ___/\   __\  / __ |/ __ \| __ \   __\
;;   |    |   |   Y  \  ___/  |  |__/ __ \_\___ \  |  |   / /_/ \  ___/| \_\ \  |  
;;   |____|   |___|  /\___  > |____(____  /____  > |__|   \____ |\___  >___  /__|  
;;                 \/     \/            \/     \/              \/    \/    \/      
;;
;;  Code Copyright (C) 2018 RetroJAM Games (@RetroJamGames)
;;  This file is part of The Last Debt: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "bullet.h.s"
.include "entity.h.s"
.include "hero.h.s"
.include "cpctelera.h.s"
.include "utils.h.s"
.include "enemy.h.s"
.include "object.h.s"
.include "buffer.h.s"
.include "levels.h.s"



;;=========================
;; Draws the object on the map
;;========================
draw_object::
	ld   a, (m_back_buffer) ;; DE = Pointer to start of the screen
  ld d, a
  ld e, #0
  ld    b, h_y(ix)                 ;; B = y coordinate (24 = 0x18)
  ld    c, h_x(ix)                  ;; C = x coordinate (16 = 0x10)
  call cpct_getScreenPtr_asm  
   ex    de,   hl
   ld h, h_sp_h(ix)
   ld l, h_sp_l(ix)
    ld b, h_h(ix)
    ld c, h_w(ix)
    call cpct_drawSprite_asm
  ret

;;=========================
;;
;;========================
pick_object:
  ld    hl, #object_here  ;;We run this method over all objects on the vector
  ld    a,  (m_num_object)                    ;; Load in A the number of the Floors
  ld    bc,  (object_vector)              ;; Load in BC pointer to the Floors vector
  ld    de, #k_obj_size
  call hero_doForAll
 
ret


;;=========================
;;
;;========================

object_doForAll::
  ld a, (m_num_object)
  cp #0
  jr z,finishO
  ld ix, (object_vector)
  ld (met), hl

  buc:
  push  af

  ld  a, (current_view)
  sub h_vw(ix)
  jr nz, here_obj

  met = . + 1
  call  met


  here_obj:
  pop af
  ld  bc, #k_obj_size    ;;|
  add   ix,   bc              ;;\ Sumar al puntero el tamaño de la entidad
  dec   a
  jr    nz,   buc
  finishO:
  ret



;;============================
;; Calculates if there's any object at our left or right
;;============================
object_here:
  call just_check

  cp #4
  ret nz

    call activate_object
    ret


;;================================
;; Copies the bullet values
;; DESTROYS: BC, HL, DE
;;================================
init_objects:

  ld hl, #draw_object 
  call object_doForAll
  call switchBuffers

  ld hl, #draw_object 
  call object_doForAll
  call switchBuffers

  ret

draw_objects:
  ld hl, #draw_object 
  call object_doForAll
ret

;;================================
;; Activates object's utility
;; 
;;================================
activate_object:

  ld a, o_status(ix)  ;;1,2,3 
  cp #0
  ret z

  ld iy, #hero
  ld a, o_type(ix)  ;;1,2,3 
  cp #1 
  jr z, add_life 
  cp #2
  jr z, add_bullet 
  jp add_shield
  add_life:
    ld a, (hearts_remaining)
    cp #5
    ret z
    add #1
    ld (hearts_remaining), a
    jr finish_picking
  add_bullet:

    ld a, h_bull(iy)
    cp #20
    ret z

    ld b, #5
    bullet_add_buc:

      add #1
      ld h_bull(iy), a
      cp #20
      jp z, finish_adding
      dec b
      jp nz, bullet_add_buc

    finish_adding:

    jr finish_picking
  add_shield:
    ld a, h_shld(iy)
    cp #1
    ret z

    add #1 
    ld h_shld(iy),a

    jr finish_picking

  finish_picking:

  ld hl, #_empty
  call change_sprite

  call draw_hud
  call draw_object
  call switchBuffers
  call draw_hud
  call draw_object
  call switchBuffers

  ld a, #0
  ld o_status(ix), a

  ret
